// IkWnd.cpp: implementation of the CIkWnd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#if defined(WIN32)
#include "IkWnd.h"

#include <algorithm>

// static 
char CIkWnd::ms_szWndClassName[32] = {0};

void CIkWnd::sRegisterWindowClass()
{
	WNDCLASSEX wc;
	wc.cbSize=sizeof wc;

	if( ms_szWndClassName[0] == '\0' )
	{
		char szModuleFileName[MAX_PATH+1];
		
		GetModuleFileName(g_hInst, szModuleFileName, MAX_PATH);

		char * pszFilename = strrchr( szModuleFileName, '\\' );
		pszFilename++;

		_snprintf(ms_szWndClassName, sizeof(ms_szWndClassName), "%s helper window for gametalk", pszFilename);
	}

	if (GetClassInfoEx(NULL, ms_szWndClassName, &wc))
		return;        /*  Already registered  */
    	
	wc.cbSize=sizeof wc;

	/*
    *  CS_GLOBALCLASS style means that this window  class is
    *  an application global class.  An application global class is
    *  a  window class registered by an executable or dynamic-link
    *  library (.dll) that is available to all other modules in the
    *  process.  ...A process that loads the .dll can create instances
    *  of the custom control. (MSDN)
    */
    wc.style = CS_GLOBALCLASS;

	wc.lpfnWndProc = CIkWnd::sWindowProc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	
	/*  Handle to the instance that the window procedure of this
    *  class is within. */
    wc.hInstance = NULL;    /*  Global class  */

	wc.hIcon = NULL;
    wc.hIconSm = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszMenuName = 0;
	wc.lpszClassName = ms_szWndClassName;
	
	RegisterClassEx(&wc);
}

void CIkWnd::sUnRegisterWindowClass()
{
	if( !ms_szWndClassName )
		return;

	UnregisterClass(
		ms_szWndClassName,  // class name
		NULL   // handle to application instance
		);

	ms_szWndClassName[0] = '\0';
}

LRESULT CALLBACK CIkWnd::sWindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	CIkWnd * pWnd = reinterpret_cast<CIkWnd*>( GetWindowLong(hWnd, GWL_USERDATA) );

	if( pWnd )
	{
		pWnd->OnWindowMsg( message, wParam, lParam );
	}

	return DefWindowProc( hWnd, message, wParam, lParam );
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIkWnd::CIkWnd(_CIkWndHandler * pHandler) : m_hWnd(NULL), m_pHandler(pHandler)
{
	sRegisterWindowClass();
}

CIkWnd::~CIkWnd()
{
	DestroyWindow();
}

BOOL CIkWnd::Create(LPCSTR szWindowName, 
					HWND hWndParent, 
					UINT uID, 
					LPRECT lpRect, 
					DWORD dwStyle, 
					DWORD dwExStyle)
{
	if( dwStyle == 0 )
	{
		if( hWndParent == NULL )
			dwStyle = WS_POPUP;
		else
			dwStyle = WS_CHILD;
	}

	m_hWnd=CreateWindowEx( dwExStyle, ms_szWndClassName, szWindowName, dwStyle, 
		lpRect ? lpRect->left : 0,
		lpRect ? lpRect->top : 0,
		lpRect ? lpRect->right - lpRect->left : 0,
		lpRect ? lpRect->bottom - lpRect->top : 0,
		hWndParent, (HMENU)uID, GetModuleHandle(0), NULL );

	if( ::IsWindow( m_hWnd ) == FALSE )
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	SetWindowLong(m_hWnd, GWL_USERDATA, (LONG)this);

	return TRUE;
}

BOOL CIkWnd::DestroyWindow()
{
	//Destroy window
	if (m_hWnd)
	{
		::DestroyWindow(m_hWnd);
		m_hWnd=NULL;
	}

	return TRUE;
}

void CIkWnd::Invalidate(BOOL bErase )
{
	::InvalidateRect( m_hWnd, NULL, bErase );
}

int CIkWnd::PostMessage(UINT message, WPARAM wParam, LPARAM lParam)
{
	return ::PostMessage( m_hWnd, message, wParam, lParam );
}

LRESULT CALLBACK CIkWnd::OnWindowMsg( UINT message, WPARAM wParam, LPARAM lParam )
{
	m_pHandler->IkWnd_OnMessage( this, message, wParam, lParam );

	return 0;
}

#endif